package de.uni_hildesheim.sse.model_extender;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;

import de.uni_hildesheim.sse.model.varModel.Project;
import de.uni_hildesheim.sse.model_extender.convert.ConstraintParserException;
import de.uni_hildesheim.sse.model_extender.convert.ModelExtender;
import de.uni_hildesheim.sse.model_extender.in.DimacsReader;
import de.uni_hildesheim.sse.model_extender.in.MalformedFileException;
import de.uni_hildesheim.sse.trans.out.DimacsWriter;

/**
 * Main class for running the ModelExtender.
 * 
 * @author Adam Krafczyk
 * @author El-Sharkawy
 */
public class Main {
    
	public static void extendModel(BufferedReader sourceDimacsFile, String constraint, File destinationDimacsFile)
	        throws FileNotFoundException, IOException, MalformedFileException, ConstraintParserException {
	        
	        // Read existing model
	        DimacsReader reader = null;
	        Project model = null;
	        try {
	            reader = new DimacsReader(sourceDimacsFile);
	            model = reader.getModel();
	        } finally {
	            if (null != reader) {
	                reader.close();
	            }
	        }
	        if (null != model) {
	            // Extend model
	            ModelExtender extender = new ModelExtender(model);
	            extender.addConstraint(constraint);
	            
	            // Saved extended model
	            FileWriter fWriter = new FileWriter(destinationDimacsFile);
	            DimacsWriter writer = new DimacsWriter(model, fWriter);
	            writer.write();
	            fWriter.close();
	        }
	    }
	
    /**
     * Will extends the <tt>sourceDimacsFile</tt> with the given <tt>constraint</tt> and save the extended file to
     * <tt>destinationDimacsFile</tt>.
     * @param sourceDimacsFile The translated KConfig model in dimacs format. Must not be <tt>null</tt>.
     * @param constraint The constraint to append to the already existing model.
     * @param destinationDimacsFile The destination of the extended model. The path must exist.
     * @throws FileNotFoundException If the <tt>sourceDimacsFile</tt> was not found or could not be read.
     * @throws IOException If reading the <tt>sourceDimacsFile</tt> throws an {@link IOException}
     * or if writing the <tt>destinationDimacsFile</tt> throws an {@link IOException} 
     * @throws MalformedFileException If the <tt>sourceDimacsFile</tt> is not correctly formatted.
     * @throws ConstraintParserException If the constraint is not correctly formatted.
     */
    public static void extendModel(File sourceDimacsFile, String constraint, File destinationDimacsFile)
        throws FileNotFoundException, IOException, MalformedFileException, ConstraintParserException {
        
        // Read existing model
        DimacsReader reader = null;
        Project model = null;
        try {
            reader = new DimacsReader(sourceDimacsFile);
            model = reader.getModel();
        } finally {
            if (null != reader) {
                reader.close();
            }
        }
        if (null != model) {
            // Extend model
            ModelExtender extender = new ModelExtender(model);
            extender.addConstraint(constraint);
            
            // Saved extended model
            FileWriter fWriter = new FileWriter(destinationDimacsFile);
            DimacsWriter writer = new DimacsWriter(model, fWriter);
            writer.write();
            fWriter.close();
        }
    }

    /**
     * Main-Method to run the ModelExtender.
     * 
     * @param args Command line parameters:
     * <ol>
     *   <li>Path to .dimacs file</li>
     *   <li>Additional constraint</li>
     *   <li>Path to destination .dimacs file</li>
     * </ol>
     */
    public static void main(String[] args) {
        if (args.length < 3) {
            System.err.println("Not enough command line arguments");
            System.err.println("Usage: path/to/in/model.dimacs \"a && b<7\" path/to/out/model.dimacs");
            System.exit(1);
        }
        
        File readModelFile = new File(args[0]);
        DimacsReader reader = null;
        try {
            reader = new DimacsReader(readModelFile);
        } catch (FileNotFoundException e) {
            System.err.println("File " + readModelFile.getPath() + " can't be opened for reading: "
                    + e.getMessage());
            System.exit(1);
        }
        
        Project model = null;
        try {
            model = reader.getModel();
        } catch (IOException | MalformedFileException e) {
            e.printStackTrace(System.err);
            System.exit(1);
        }
        
        try {
            reader.close();
        } catch (IOException e) {
            e.printStackTrace(System.err);
            System.exit(1);
        }
        
        ModelExtender extender = new ModelExtender(model);
        try {
            extender.addConstraint(args[1]);
            
        } catch (ConstraintParserException e) {
            e.printStackTrace(System.err);
        }
        
        File writeModelFile = new File(args[2]);
        try {
            DimacsWriter writer = new DimacsWriter(model, new FileWriter(writeModelFile));
            writer.write();
        } catch (IOException e) {
            e.printStackTrace(System.err);
        }
    }

}
